﻿using UnityEngine;
using System;
using System.Collections;

public class NetworkLogic : MonoBehaviour {
	
	public string GameName = "Decoder Ring";
	public string GameType = "Candy Cravers";
	public string MasterServerIP = "127.0.0.1";

	const int Player0Tag = 1 << 8;
	const int Player1Tag = 1 << 9;

	public GameObject PlayerPrefab;
	public Transform SpawnPoint0;
	public Transform SpawnPoint1;

	public bool AutoSpawn = false;

	GameObject[] LocalPlayers;

	HostData[] VisibleHosts;
	bool bWaitingForServerList;
	string LastServerEvent = "";
	bool bSplitScreenStarted;

	float ButtonX;
	float ButtonY;
	float ButtonWidth;
	float ButtonHeight;
	float TextFieldHeight;
	float ButtonSpacing;

	// Use this for initialization
	void Start () {
		ButtonX = Screen.width * 0.05f;
		ButtonY = Screen.width * 0.05f;
		ButtonWidth = Screen.width * 0.13f;
		ButtonHeight = Screen.width * 0.1f;
		TextFieldHeight = ButtonHeight * 0.25f;
		ButtonSpacing = ButtonHeight * 0.2f;
		MasterServer.ipAddress = MasterServerIP;

		if (AutoSpawn)
		{
			if (Network.isServer)
				SpawnPlayer(new Vector3(1,0,0));
			else if (Network.isClient)
				SpawnPlayer(new Vector3(0,0,1));
			else StartSplitScreen();
		}
	}
	
	// Update is called once per frame
	void Update () {
		if (bWaitingForServerList)
		{
			VisibleHosts = MasterServer.PollHostList();
			if (VisibleHosts.Length > 0)
			{
				bWaitingForServerList = false;
				Debug.Log("Received host list of " + VisibleHosts.Length + " servers from " + MasterServerIP);
			}

		}
	}
	
	public void SpawnPlayer(Vector3 color)
	{
		Transform spawnPoint = !Network.isClient && null == LocalPlayers ? SpawnPoint0 : SpawnPoint1;
		GameObject player = null;

		if (bSplitScreenStarted)
		{
			player = (GameObject)Instantiate(PlayerPrefab);
			player.transform.position = spawnPoint.position;
			player.transform.rotation = spawnPoint.rotation;
		}
		else
		{
			player = (GameObject)Network.Instantiate (PlayerPrefab, spawnPoint.transform.position, spawnPoint.transform.rotation, 0);
			player.SendMessage("SetNetworked");
			player.transform.GetComponentInChildren(typeof(MouseLook)).SendMessage("SetNetworked");
		}
		if (null != player.renderer)
		{
			player.renderer.material.color = new Color (color.x, color.y, color.z, 1);
		}

		Transform gfxXform = player.transform.FindChild ("Graphics");
		if (null != gfxXform.renderer)
		{
			gfxXform.renderer.material.color = new Color (color.x, color.y, color.z, 1);
		}
		
		if (null == LocalPlayers)
			LocalPlayers = new GameObject[1];
		else
		{
			Array.Resize(ref LocalPlayers, LocalPlayers.Length + 1);
		}
		LocalPlayers [LocalPlayers.Length - 1] = player;

		player.SendMessage ("SetPlayerNumber", LocalPlayers.Length, SendMessageOptions.DontRequireReceiver);
	}

	void StartSplitScreen()
	{
		bSplitScreenStarted = true;
		SpawnPlayer (new Vector3 (1, 0, 0));
		SpawnPlayer (new Vector3 (0, 0, 1));
		SetupSplitScreenCameras ();
	}
	
	void SetupSplitScreenCameras()
	{
		if (bSplitScreenStarted)
		{
			for (int p = 0; p < LocalPlayers.Length; p++)
			{
				GameObject player = LocalPlayers[p];
				Camera cam = player.GetComponentInChildren<Camera>();
				cam.rect = new Rect(p / (float)(LocalPlayers.Length), 0, 1.0f / LocalPlayers.Length, 1.0f);
				if (p == 0)
				{
					cam.cullingMask &= ~Player1Tag;
					cam.cullingMask |= Player0Tag;
				}
				else
				{
					cam.cullingMask &= ~Player0Tag;
					cam.cullingMask |= Player1Tag;
				}
				cam.SendMessage ("SetPlayerNumber", (p + 1), SendMessageOptions.DontRequireReceiver);
			}
		}
	}

	void StartServer()
	{
		bool bPubliclyVisible = Network.HavePublicAddress();
		Network.InitializeServer(3, 17342, !bPubliclyVisible);
		MasterServer.RegisterHost(GameType, GameName);
	}

	void OnServerInitialized()
	{
		Debug.Log ("Server initialized");
		SpawnPlayer (new Vector3(1,0,0));
		SetWorldVisibility ();
	}

	void OnMasterServerEvent(MasterServerEvent evt)
	{
		if (evt == MasterServerEvent.RegistrationSucceeded)
		{
			Debug.Log("Registered Server");
		}
		else
		{
			LastServerEvent = evt.ToString();
		}
	}

	void OnConnectedToServer()
	{
		Debug.Log ("Connected to server");
		SpawnPlayer (new Vector3(0,0,1));
		SetWorldVisibility ();
	}

	void RefreshHostList()
	{
		Debug.Log ("Requesting host list");
		MasterServer.RequestHostList(GameType);
		bWaitingForServerList = true;
		LastServerEvent = "";
	}

	void SetWorldVisibility()
	{
		GameObject p0World = GameObject.Find ("Player0Objects");
		GameObject p1World = GameObject.Find ("Player1Objects");

		SetHierarchyVisibility (p0World, Network.isServer);
		SetHierarchyVisibility (p1World, Network.isClient);
	}

	void SetHierarchyVisibility(GameObject root, bool bVisible)
	{
		if (null != root.renderer)
			root.renderer.enabled = bVisible;
		Debug.Log ("Setting visibility on " + root.name + " to " + bVisible.ToString ());
		foreach(Transform child in root.transform)
		{
			SetHierarchyVisibility(child.gameObject, bVisible);
		}
	}

	void OnGUI()
	{
		if (!bSplitScreenStarted && !Network.isClient && !Network.isServer)
			DisplayConnectionButtons ();
	}

	void DisplayConnectionButtons()
	{
		float y = ButtonY;
		
		GUI.TextArea (new Rect (ButtonX, y, ButtonWidth * 3, TextFieldHeight), "Candygram Afghanistan!!:");
		
		y += TextFieldHeight + ButtonSpacing;
		
		if (GUI.Button(new Rect(ButtonX, y, ButtonWidth, ButtonHeight), "Start Split Screen"))
		{
			StartSplitScreen();
		}
		
		y += ButtonHeight + ButtonSpacing;
		
		GUI.TextArea (new Rect (ButtonX, y, ButtonWidth, TextFieldHeight), "Master Server IP:");
		
		MasterServerIP = GUI.TextField(new Rect(ButtonX + ButtonWidth + ButtonSpacing, y, ButtonWidth, TextFieldHeight), MasterServerIP);
		
		y += TextFieldHeight + ButtonSpacing;

		if (GUI.Button(new Rect(ButtonX, y, ButtonWidth, ButtonHeight), "Start Server"))
		{
			StartServer();
		}

		y += ButtonHeight + ButtonSpacing;

		if (GUI.Button (new Rect(ButtonX, y, ButtonWidth, ButtonHeight), "Refresh Servers"))
		{
			RefreshHostList();
		}

		if (LastServerEvent.Length > 0)
		{
			y += ButtonHeight + ButtonSpacing;
			GUI.TextArea (new Rect (ButtonX, y, ButtonWidth, TextFieldHeight), LastServerEvent);
		}

		if (null != VisibleHosts)
		{
			y = ButtonY + TextFieldHeight + ButtonSpacing;

			for (int h = 0; h < VisibleHosts.Length; h++)
			{
				y += ButtonHeight + ButtonSpacing;
				HostData host = VisibleHosts[h];
				if (GUI.Button(new Rect(ButtonX + ButtonWidth + ButtonSpacing, y, ButtonWidth, ButtonHeight), VisibleHosts[h].gameName))
				{
					Network.Connect(host);
				}
			}	
		}
	}
}
